/*
 * File: system_init.c
 * Date: 31.08.2012
 * Denis Zheleznyakov http://ziblog.ru
 */

#include "main.h"

//------------------------------------------------------------------------------
inline static void gpio(void)
{
	// включение тактирования
	RCC_APB2PeriphClockCmd(
			RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOC
					| RCC_APB2Periph_GPIOD | RCC_APB2Periph_GPIOE, ENABLE);

	// светодиоды
	PIN_CONFIGURATION(PIN_LED1);
	PIN_CONFIGURATION(PIN_LED2);
	PIN_CONFIGURATION(PIN_LED3);
	PIN_CONFIGURATION(PIN_LED4);

	// кнопки
	PIN_CONFIGURATION(PIN_BUTTON1);
	PIN_CONFIGURATION(PIN_BUTTON2);
	PIN_CONFIGURATION(PIN_BUTTON3);
	PIN_CONFIGURATION(PIN_BUTTON4);

	// USB
	PIN_CONFIGURATION(PIN_USB_PUP);

	// выход MCO
	PIN_CONFIGURATION(PIN_MCO);
}

//------------------------------------------------------------------------------
inline static void interrupt(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;

	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

	NVIC_InitStructure.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
}

//------------------------------------------------------------------------------
inline static void usb(void)
{
	// выбор источника тактовых сигналов
	RCC_USBCLKConfig(RCC_USBCLKSource_PLLCLK_Div1);

	// включение тактирования
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USB, ENABLE);

	//
	USB_Init();
}

//------------------------------------------------------------------------------
inline static void clock(void)
{
	// включаем HSI
	RCC->CR |= RCC_CR_HSION;

	// сброс - SW, HPRE, PPRE1, PPRE2, ADCPRE, MCO
	RCC->CFGR &= ~(RCC_CFGR_SW | RCC_CFGR_HPRE | RCC_CFGR_PPRE1
			| RCC_CFGR_PPRE2 | RCC_CFGR_ADCPRE | RCC_CFGR_MCO);

	// сброс - HSEON, CSSON, PLLON
	RCC->CR &= ~(RCC_CR_HSEON | RCC_CR_CSSON | RCC_CR_PLLON);

	// сброс - HSEBYP
	RCC->CR &= ~(RCC_CR_HSEBYP);

	// сброс - PLLSRC, PLLXTPRE, PLLMUL, USBPRE/OTGFSPRE
	RCC->CFGR &= ~(RCC_CFGR_PLLSRC | RCC_CFGR_PLLXTPRE | RCC_CFGR_PLLMULL
			| RCC_CFGR_USBPRE);

	// сброс и запрет всех прерываний
	RCC->CIR = 0x009F0000;

	// ремап таблицы векторов прерываний во флеш-память
	SCB->VTOR = FLASH_BASE;

	// запуск HSE генератора
	RCC_HSEConfig(RCC_HSE_ON);
	while (!RCC_GetFlagStatus(RCC_FLAG_HSERDY))
	{
	}

	// переключаемся
	RCC_SYSCLKConfig(RCC_SYSCLKSource_HSE);
	while (RCC_GetSYSCLKSource() != RCC_CFGR_SWS_HSE)
	{
	}

	// ФАПЧ для тактирования USB
	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_6);
	RCC_PLLCmd(ENABLE);
	while (!RCC_GetFlagStatus(RCC_FLAG_PLLRDY))
	{
	}

	RCC_MCOConfig(RCC_MCO_PLLCLK_Div2);

}

//------------------------------------------------------------------------------
void SystemInit(void)
{
	clock();

	gpio();

	interrupt();

	usb();
}
